////// Fragment Shader
precision highp float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform mat4 mBlendMatrix;
uniform int mCutoutType;

vec4 getShowColorWithTranslucenta(vec4 srcColor, vec4 maskColor) {
    vec4 translucentaColor = vec4(0, 0, 0, 1.0);
    vec4 blendColor = translucentaColor * (1.0 - maskColor.a) * 0.5;
    return srcColor * (1.0 - blendColor.a) + blendColor;
}

void main()
{
    vec4 srcColor = texture2D(inputImageTexture, textureCoordinate);
    vec4 coordinate = vec4(textureCoordinate2, 0., 1.);
    coordinate = mBlendMatrix * coordinate;
    vec4 maskColor = texture2D(inputImageTexture2, coordinate.xy);
    if (mCutoutType == 1 && (coordinate.x < 0.0 || coordinate.x > 1.0 || coordinate.y < 0.0 || coordinate.y > 1.0)) {
        gl_FragColor = srcColor;
        return;
    }
    if (mCutoutType == 2) {
        // shape 裁剪+预览
        gl_FragColor = getShowColorWithTranslucenta(srcColor, maskColor);
    } else if (mCutoutType == 3) {
        // 当前的透明度效果
        vec4 translucenta = vec4(srcColor.rgb * 0.6 + (1. - srcColor.rgb) * 0.3, 0.85);
        // 计算混合因子
        float mixFactor =(1.-maskColor.a) * srcColor.a;
        // 混合源颜色和透明颜色
        gl_FragColor = srcColor * maskColor.a + translucenta * mixFactor;
    } else if (mCutoutType == 4 || mCutoutType == 1) {
        gl_FragColor = srcColor * maskColor.a;
    } else {
        gl_FragColor = srcColor;
    }
}